<?php
/**
 * @version		$Id: sitemaps.php 14 2010-07-02 07:19:03Z guille $
 * @copyright           Copyright (C) 2007 - 2009 Joomla! Vargas. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		Guillermo Vargas (guille@vargas.co.cr)
 */
// no direct access
defined('_JEXEC') or die;

jimport('joomla.application.component.modellist');
jimport('joomla.database.query');

/**
 * Sitemaps Model Class
 *
 * @package         Xmap
 * @subpackage      com_xmap
 * @since           2.0
 */
class XmapModelSitemaps extends JModelList
{

    /**
     * Model context string.
     *
     * @var	 string
     */
    public $_context = 'com_xmap.sitemaps';

    /**
     * Method to auto-populate the model state.
     *
     * @since       2.0
     */
    protected function _populateState()
    {
        $app = JFactory::getApplication();

        $access = $app->getUserStateFromRequest($this->_context . '.filter.access', 'filter_access', 0, 'int');
        $this->setState('filter.access', $access);

        $published = $app->getUserStateFromRequest($this->_context . '.published', 'filter_published', '');
        $this->setState('filter.published', $published);

        // List state information
        $limit = $app->getUserStateFromRequest('global.list.limit', 'limit', $app->getCfg('list_limit'));
        $this->setState('list.limit', $limit);

        $limitstart = $app->getUserStateFromRequest($this->_context . '.limitstart', 'limitstart', 0);
        $this->setState('list.limitstart', $limitstart);

        $orderCol = $app->getUserStateFromRequest($this->_context . '.ordercol', 'filter_order', 'a.title');
        $this->setState('list.ordering', $orderCol);

        $orderDirn = $app->getUserStateFromRequest($this->_context . '.orderdirn', 'filter_order_Dir', 'asc');
        $this->setState('list.direction', $orderDirn);
    }

    /**
     * @param       boolean True to join selected foreign information
     *
     * @return      string
     */
    protected function getListQuery($resolveFKs = true)
    {
        $db = JFactory::getDBO();
        // Create a new query object.
        $query = $db->getQuery(true);

        // Select the required fields from the table.
        $query->select(
                $this->getState(
                        'list.select',
                        'a.*')
        );
        $query->from('#__xmap_sitemap AS a');

        // Join over the asset groups.
        $query->select('ag.title AS access_level');
        $query->join('LEFT', '#__viewlevels AS ag ON ag.id = a.access');

        // Filter by access level.
        if ($access = $this->getState('filter.access')) {
            $query->where('a.access = ' . (int) $access);
        }

        // Filter by published state
        $published = $this->getState('filter.published');
        if (is_numeric($published)) {
            $query->where('a.state = ' . (int) $published);
        } else if ($published === '') {
            $query->where('(a.state = 0 OR a.state = 1)');
        }

        // Add the list ordering clause.
        $query->order($this->_db->getEscaped($this->getState('list.ordering', 'a.title')) . ' ' . $this->_db->getEscaped($this->getState('list.direction', 'ASC')));

        //echo nl2br(str_replace('#__','jos_',$query));
        return $query;
    }

}
